@@ -157,10 +157,12 @@ bool BitBoard::isCheckmate() const
 {
     MoveList moves(generateMoves());
     for(int i = 0; i < moves.size(); ++i)
+    {
         if(!isIntoCheck(moves[i]))
         {
             return false;
         }
+    }
     return isCheck();
 }
 
@@ -168,10 +170,12 @@ bool BitBoard::isStalemate() const
 {
     MoveList moves(generateMoves());
     for(int i = 0; i < moves.size(); ++i)
+    {
         if(!isIntoCheck(moves[i]))
         {
             return false;
         }
+    }
     return !isCheck();
 }
 
@@ -240,7 +244,7 @@ QString BitBoard::moveToSan(const Move& move, bool translate) const
 
     if(move.isCastling())
     {
-        if(from < to)
+        if (File(to)==File(g1))
         {
             san = "O-O";
         }
@@ -1213,10 +1217,10 @@ void BitBoard::fixCastlingRooks(bool onlyMissingSections, char file000, char fil
 
     Square filesForSection[4] =
     {
-        file000 ? SquareFromRankAndFile(0,file000-'a') : InvalidSquare,
-        file00 ? SquareFromRankAndFile(0,file00-'a') : InvalidSquare,
-        file000 ? SquareFromRankAndFile(7,file000-'a') : InvalidSquare,
-        file00 ? SquareFromRankAndFile(7,file00-'a') : InvalidSquare
+        file000 ? SquareFromRankAndFile(0,file000-'a') : Square(InvalidSquare),
+        file00 ? SquareFromRankAndFile(0,file00-'a') : Square(InvalidSquare),
+        file000 ? SquareFromRankAndFile(7,file000-'a') : Square(InvalidSquare),
+        file00 ? SquareFromRankAndFile(7,file00-'a') : Square(InvalidSquare)
     };
 
     for (int section=0; section<4; ++section)
@@ -1289,10 +1293,10 @@ bool BitBoard::hasAmbiguousCastlingRooks(char file000, char file00) const
 
     Square filesForSection[4] =
     {
-        file000 ? SquareFromRankAndFile(0,file000-'a') : InvalidSquare,
-        file00 ? SquareFromRankAndFile(0,file00-'a') : InvalidSquare,
-        file000 ? SquareFromRankAndFile(7,file000-'a') : InvalidSquare,
-        file00 ? SquareFromRankAndFile(7,file00-'a') : InvalidSquare
+        file000 ? SquareFromRankAndFile(0,file000-'a') : Square(InvalidSquare),
+        file00 ? SquareFromRankAndFile(0,file00-'a') : Square(InvalidSquare),
+        file000 ? SquareFromRankAndFile(7,file000-'a') : Square(InvalidSquare),
+        file00 ? SquareFromRankAndFile(7,file00-'a') : Square(InvalidSquare)
     };
 
     for (int section=0; section<4; ++section)
@@ -2878,6 +2882,24 @@ bool BitBoard::insufficientMaterial() const
     return false;
 }
 
+Color BitBoard::colorAt(Square s) const
+{
+    Q_ASSERT(s < 64);
+    quint64 bit = SetBit(s);
+    if(m_occupied & bit)
+    {
+        if(m_occupied_co[White] & bit)
+        {
+            return White;
+        }
+        else
+        {
+            return Black;
+        }
+    }
+    return NoColor;
+}
+
 Piece BitBoard::pieceAt(Square s) const
 {
     Q_ASSERT(s < 64);
@@ -2908,7 +2930,7 @@ inline QChar pieceToHumanChar(const Piece piece)
     return piece > BlackPawn ? '?' : " KQRBNPKQRBNP"[piece];
 };
 
-QString BitBoard::toFen() const
+QString BitBoard::toFen(bool forceExtendedFEN) const
 {
     QString fen = "";
     Piece piece;
@@ -2973,7 +2995,26 @@ QString BitBoard::toFen() const
         {
             s000 = 'a'+File(CastlingRook(2));
         }
-        if (!hasAmbiguousCastlingRooks(s000, s00))
+        if (forceExtendedFEN || hasAmbiguousCastlingRooks(s000, s00))
+        {
+            if(castlingRights() & WhiteQueenside)
+            {
+                fen += 'A'+File(CastlingRook(0));
+            }
+            if(castlingRights() & WhiteKingside)
+            {
+                fen += 'A'+File(CastlingRook(1));
+            }
+            if(castlingRights() & BlackQueenside)
+            {
+                fen += 'a'+File(CastlingRook(2));
+            }
+            if(castlingRights() & BlackKingside)
+            {
+                fen += 'a'+File(CastlingRook(3));
+            }
+        }
+        else
         {
             if(castlingRights() & WhiteKingside)
             {
@@ -2992,25 +3033,6 @@ QString BitBoard::toFen() const
                 fen += 'q';
             }
         }
-        else
-        {
-            if(castlingRights() & WhiteKingside)
-            {
-                fen += 'A'+File(CastlingRook(1));
-            }
-            if(castlingRights() & WhiteQueenside)
-            {
-                fen += 'A'+File(CastlingRook(0));
-            }
-            if(castlingRights() & BlackKingside)
-            {
-                fen += 'a'+File(CastlingRook(3));
-            }
-            if(castlingRights() & BlackQueenside)
-            {
-                fen += 'a'+File(CastlingRook(2));
-            }
-        }
         fen += ' ';
     }
 
@@ -3037,7 +3059,7 @@ QString BitBoard::toFen() const
 
 QString BitBoard::toHumanFen() const
 {
-    QString fenFormat = QCoreApplication::translate("BitBoard", "w%1\nb%2\n%3 to move.");
+    QString fenFormat = QCoreApplication::translate("BitBoard", "w%1\nb%2\n%3 to move");
     QMap<Piece, QStringList> charLists;
 
     //piece placement
@@ -3083,7 +3105,7 @@ QString BitBoard::toHumanFen() const
     }
 
     QString fen = fenFormat.arg(w).arg(b).arg(toMove);
-
+    fen.append(QString(" (%1+%2).").arg(m_pieceCount[White]).arg(m_pieceCount[Black]));
     return fen;
 }
 
