@@ -92,6 +92,8 @@ public:
     bool isMovable(const Square s) const;
     /** @return piece sitting at given square on the board */
     Piece pieceAt(Square s) const;
+    /* Get the color of a Piece at square @s */
+    Color colorAt(Square s) const;
     /** @return number of ply since a pawn move or capture */
     unsigned int halfMoveClock() const;
     /** Set number of ply since a pawn move or capture */
@@ -110,7 +112,7 @@ public:
     // Query other formats
     //
     /** Return a FEN string based on current board position */
-    QString toFen() const;
+    QString toFen(bool forceExtendedFEN=false) const;
     /** Return a FEN string in human readable format based on current board position */
     QString toHumanFen() const;
     /** Return a SAN string representation of given move */
@@ -160,6 +162,8 @@ public:
     bool canCastleLong(const unsigned int color) const;
     /** Get the castling rook to the given index into m_castlingRook */
     Square CastlingRook(int index) const;
+    /** Return the square of the king if in check, InvalidSquare otherwise */
+    Square kingInCheck() const;
 protected:
     unsigned int countSetBits(quint64 n) const;
 private:
@@ -167,6 +171,7 @@ private:
     bool isKingOnRow(Piece p, Square start, Square stop) const;
     /** Return true if side to move is in check */
     bool isCheck() const;
+
     /** Test to see if given color has any castling rights remaining */
     bool canCastle(const unsigned int color) const;
 
@@ -450,6 +455,11 @@ inline bool BitBoard::isCheck() const
     return isAttackedBy(m_stm ^ 1, m_ksq[m_stm]);
 }
 
+inline Square BitBoard::kingInCheck() const
+{
+    return isCheck() ? m_ksq[m_stm] : (Square) InvalidSquare;
+}
+
 inline unsigned int BitBoard::halfMoveClock() const
 {
     return m_halfMoves;
