@@ -12,6 +12,7 @@
 
 #include <QtCore>
 #include <QDialog>
+#include <QDockWidget>
 #include <QtGui>
 #include <QWidget>
 #include <QMainWindow>
@@ -45,14 +46,22 @@ bool Settings::layout(QWidget* w)
     if(valid)    // Enough values
     {
         int x = values[0];
-        x &= ~0xC0000000;
-        if (values[0] & 0x80000000)      w->setWindowState(Qt::WindowMaximized);
-        else if (values[0] & 0x40000000) w->setWindowState(Qt::WindowFullScreen);
+        int y = values[1];
+        int wx;
+        if (values.count()==5)
+        {
+            wx = x;
+            x &= ~0xC0000000;
+        }
         else
         {
-            w->resize(values[2], values[3]);
-            w->move(x, values[1]);
+            wx = values[5];
         }
+        w->resize(values[2], values[3]);
+        w->move(x, y);
+
+        if (wx & 0x80000000)      w->setWindowState(Qt::WindowMaximized);
+        else if (wx & 0x40000000) w->setWindowState(Qt::WindowFullScreen);
 
         QSplitter* s = qobject_cast<QSplitter*>(w);
         if (s)
@@ -73,6 +82,19 @@ bool Settings::layout(QWidget* w)
                 {
                     m->restoreState(docks, 0);
                 }
+
+                QStringList floatingDocks = value("FloatingDocks").toStringList();
+                foreach(QString name, floatingDocks)
+                {
+                    QDockWidget* dw = m->findChild<QDockWidget*>(name);
+                    if (dw)
+                    {
+                        dw->setFloating(true);
+                        QRect rect = value(QString("FloatingDock/%1/geometry").arg(name)).toRect();
+                        dw->setGeometry(rect);
+                        dw->setVisible(value(QString("FloatingDock/%1/visible").arg(name)).toBool());
+                    }
+                }
             }
             else
             {
@@ -105,16 +127,48 @@ void Settings::setLayout(const QWidget* w)
     }
     beginGroup("Geometry");
     QList<int> values;
+    int wx = 0;
     int x = w->x();
-    if (w->isFullScreen())     x |= 0x40000000;
-    else if (w->isMaximized()) x |= 0x80000000;
-    values << x << w->y() << w->width() << w->height() << w->isVisible();
+    int y = w->y();
+    int wn = w->width();
+    int hn = w->height();
+    if (w->isFullScreen())     wx |= 0x40000000;
+    else if (w->isMaximized()) wx |= 0x80000000;
+
+    if (wx)
+    {
+        QRect ng = w->normalGeometry();
+        QRect frame = w->frameGeometry();
+        ng.adjust(-frame.x(), -frame.y(), -frame.x(), -frame.y());
+        x = ng.x();
+        y = ng.y();
+        wn = ng.width();
+        hn = ng.height();
+    }
+    values << x << y << wn << hn << w->isVisible() << wx;
     setList(w->objectName(), values);
     const QMainWindow* m = qobject_cast<const QMainWindow*>(w);
     if (m)
     {
         QByteArray docks = m->saveState(0);
         setValue("Docks", docks);
+
+        QStringList floatingDocks;
+        QList<QDockWidget*> fdocks = m->findChildren<QDockWidget*>();
+        for(int i = 0; i < fdocks.size(); i++)
+        {
+            QDockWidget* dw = fdocks.at(i);
+            if (dw->isFloating())
+            {
+                QString name = dw->objectName();
+                floatingDocks << name;
+                QRect rect = dw->geometry();
+                setValue(QString("FloatingDock/%1/geometry").arg(name), rect);
+                setValue(QString("FloatingDock/%1/visible").arg(name), !dw->isHidden());
+            }
+        }
+
+        setValue("FloatingDocks", floatingDocks);
     }
     const QSplitter* s = qobject_cast<const QSplitter*>(w);
     if (s)
@@ -239,7 +293,7 @@ void Settings::setList(const QString& key, QList<int> list)
 bool Settings::list(const QString &key, QList<int>& list, int items)
 {
     QList<QVariant> varlist = value(key).toList();
-    if(items >= 0 && varlist.count() + list.count() != items)
+    if(items >= 0 && varlist.count() + list.count() < items)
     {
         return false;
     }
@@ -341,6 +395,7 @@ QMap<QString, QVariant> Settings::initDefaultValues() const
     map.insert("/Board/showMoveIndicator", 0);
     map.insert("/Board/guessMove", true);
     map.insert("/Board/showThreat", true);
+    map.insert("/Board/showTargets", false);
     map.insert("/Board/noHints", false);
     map.insert("/Board/nextGuess", false);
     map.insert("/Board/minWheelCount", MIN_WHEEL_COUNT);
@@ -354,6 +409,8 @@ QMap<QString, QVariant> Settings::initDefaultValues() const
     map.insert("/Board/currentMoveColor", QColor(Qt::blue));
     map.insert("/Board/storedMoveColor", QColor(Qt::magenta));
     map.insert("/Board/threatColor", QColor(0xe0,0xe0,0xff,0x80));
+    map.insert("/Board/targetColor", QColor(Qt::darkGreen));
+    map.insert("/Board/checkColor", QColor(Qt::magenta));
     map.insert("/Board/AutoPlayerInterval", 3000);
     map.insert("/Board/AutoSaveAndContinue", false);
     map.insert("/Board/BackwardAnalysis", false);
