@@ -455,7 +455,6 @@ void PreferencesDialog::restoreSettings()
     ui.cbSaveAndContinue->setChecked(AppSettings->getValue("AutoSaveAndContinue").toBool());
     ui.cbBackwardAnalysis->setChecked(AppSettings->getValue("BackwardAnalysis").toBool());
     ui.cbPromoteToQueen->setChecked(AppSettings->getValue("AutoPromoteToQueen").toBool());
-    ui.btShowThreat->setChecked(AppSettings->getValue("showThreat").toBool());
     ui.btNoHints->setChecked(AppSettings->getValue("noHints").toBool());
 
     QString pieceTheme = AppSettings->getValue("pieceTheme").toString();
@@ -470,6 +469,8 @@ void PreferencesDialog::restoreSettings()
     restoreColorItem(ui.boardColorsList, tr("Current move"), "currentMoveColor");
     restoreColorItem(ui.boardColorsList, tr("Stored move"), "storedMoveColor");
     restoreColorItem(ui.boardColorsList, tr("Threat"), "threatColor");
+    restoreColorItem(ui.boardColorsList, tr("Targets"), "targetColor");
+    restoreColorItem(ui.boardColorsList, tr("Check"), "checkColor");
     AppSettings->endGroup();
 
     SlotPieceEffectActivated(ui.pieceEffect->currentIndex());
@@ -582,7 +583,6 @@ void PreferencesDialog::saveSettings()
     AppSettings->setValue("showCurrentMove", QVariant(ui.hilightCurrentMove->currentIndex()));
     AppSettings->setValue("showMoveIndicator", QVariant(ui.cbShowIndicator->currentIndex()));
     AppSettings->setValue("guessMove", QVariant(ui.guessMoveCheck->isChecked()));
-    AppSettings->setValue("showThreat", QVariant(ui.btShowThreat->isChecked()));
     AppSettings->setValue("noHints", QVariant(ui.btNoHints->isChecked()));
     AppSettings->setValue("nextGuess", QVariant(ui.guessNextMove->isChecked()));
     AppSettings->setValue("minWheelCount", ui.minWheelCount->value());
@@ -600,7 +600,8 @@ void PreferencesDialog::saveSettings()
         AppSettings->setValue("boardTheme", QString());
     }
     QStringList colorNames;
-    colorNames << "lightColor" << "darkColor" << "highlightColor" << "frameColor" << "currentMoveColor" << "storedMoveColor" << "threatColor";
+    colorNames << "lightColor" << "darkColor" << "highlightColor"
+               << "frameColor" << "currentMoveColor" << "storedMoveColor" << "threatColor" << "targetColor" << "checkColor";
     saveColorList(ui.boardColorsList, colorNames);
     AppSettings->endGroup();
 
