@@ -942,7 +942,6 @@ Position::Init(void)
     Board [COLOR_SQUARE] = EMPTY;
     Board [NULL_SQUARE] = END_OF_BOARD;
     LegalMoves = NULL;
-    SANStrings = NULL;
 
     // Make sure all tables used for move generation, hashing,
     // square tests, etc have been computed:
@@ -3185,7 +3184,7 @@ Position::MakeSANString(simpleMoveT * m, char * s, sanFlagT flag)
             // SHOULD handle 3-way ambiguity!  Looks like it does ok.
             bool ambiguous_fyle = false;
             bool ambiguous_rank = false;
-            char r, f, r2, f2;
+            char r, f, f2;
             f = square_FyleChar(from);
             r = square_RankChar(from);
             MoveList mlist;
@@ -3200,7 +3199,6 @@ Position::MakeSANString(simpleMoveT * m, char * s, sanFlagT flag)
                 {
                     /* we have an ambiguity */
                     f2 = square_FyleChar(from2);
-                    r2 = square_RankChar(from2);
                     if(f == f2)     // ambiguous fyle, so print rank
                     {
                         ambiguous_fyle = true;
