@@ -89,6 +89,8 @@ bool BoardTheme::loadPieces(const QString& pieces, int effect)
         return false;
     }
 
+    big.setDevicePixelRatio(2.0);
+
     int realsize = big.height() / 2;
     if(realsize == big.width() / 6)
     {
@@ -181,6 +183,8 @@ void BoardTheme::configure()
     setColor(BoardTheme::CurrentMove, AppSettings->getValue("currentMoveColor").value<QColor>());
     setColor(BoardTheme::StoredMove, AppSettings->getValue("storedMoveColor").value<QColor>());
     setColor(BoardTheme::Threat, AppSettings->getValue("threatColor").value<QColor>());
+    setColor(BoardTheme::Target, AppSettings->getValue("targetColor").value<QColor>());
+    setColor(BoardTheme::Check, AppSettings->getValue("checkColor").value<QColor>());
 
     QString pieceTheme = AppSettings->getValue("pieceTheme").toString();
     int pieceEffect = AppSettings->getValue("pieceEffect").toInt();
@@ -220,7 +224,9 @@ void BoardTheme::setSize(const QSize& value)
     m_size = value;
     for(int i = 1; i < ConstPieceTypes; i++)
     {
-        m_piece[i] = m_originalPiece[i].scaled(m_size, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+        float r = m_originalPiece[i].devicePixelRatio();
+        m_piece[i] = m_originalPiece[i].scaled(m_size * r, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+        m_piece[i].setDevicePixelRatio(r);
     }
     updateSquares();
 }
