@@ -34,7 +34,7 @@ const int MoveIndicatorSize = 12;
 
 BoardView::BoardView(QWidget* parent, int flags) : QWidget(parent),
     m_flipped(false), m_showFrame(false), m_showCurrentMove(2),
-    m_guessMove(false), m_showThreat(false), m_selectedSquare(InvalidSquare),
+    m_guessMove(false), m_showThreat(false), m_showTargets(false), m_brushMode(false), m_selectedSquare(InvalidSquare),
     m_hoverSquare(InvalidSquare),
     m_hiFrom(InvalidSquare), m_hiTo(InvalidSquare),
     m_threatFrom(InvalidSquare), m_threatTo(InvalidSquare),
@@ -44,7 +44,8 @@ BoardView::BoardView(QWidget* parent, int flags) : QWidget(parent),
     m_atLineEnd(true),
     m_flags(flags),
     m_coordinates(false), m_dragged(Empty), m_clickUsed(false), m_wheelCurrentDelta(0),
-    m_minDeltaWheel(0), m_moveListCurrent(0), m_showMoveIndicator(true), m_vAlignTop(false), m_DbIndex(0)
+    m_minDeltaWheel(0), m_moveListCurrent(0), m_showMoveIndicator(true), 
+    m_DbIndex(0)
 {
     QSizePolicy policy = sizePolicy();
     policy.setHeightForWidth(true);
@@ -104,6 +105,8 @@ void BoardView::setBoard(const Board& value, Square from, Square to, bool atLine
     m_currentTo = to;
     m_atLineEnd = atLineEnd;
     m_hiFrom = m_hiTo = InvalidSquare;
+    m_alertSquare = value.kingInCheck();
+    m_targets.clear();
     if(underMouse())
     {
         updateGuess(m_hoverSquare);
@@ -138,6 +141,11 @@ const BoardTheme& BoardView::theme() const
     return m_theme;
 }
 
+QSize BoardView::themeSize() const
+{
+    return m_theme.size();
+}
+
 void BoardView::showMoveIndicator(bool visible)
 {
     m_showMoveIndicator = visible;
@@ -161,7 +169,6 @@ bool BoardView::showCoordinates() const
 void BoardView::drawSquares(QPaintEvent* event)
 {
     QPainter p(this);
-    p.translate(m_translate);
     for (Square square=a1; square<NumSquares; ++square)
     {
         QRect rect = squareRect(square);
@@ -189,7 +196,7 @@ QRect BoardView::totalRect() const
     {
         rect2 = squareRect(isFlipped() ? a8 : h1);
     }
-    QRect totalRect = QRect(0,rect1.top(), width(), rect2.bottom()-m_translate.y()+2);
+    QRect totalRect = QRect(0,rect1.top(), width(), rect2.bottom()+2);
     return totalRect;
 }
 
@@ -244,14 +251,14 @@ void BoardView::drawMoveIndicator(QPaintEvent* event)
         if(square)
         {
             int posy = (white == m_flipped) ? 1 : 8 * m_theme.size().width() - square;
-            QRect rect(QPoint(8 * m_theme.size().width() + 2 + coord, posy) + m_translate,
+            QRect rect(QPoint(8 * m_theme.size().width() + 2 + coord, posy),
                        QSize(square, square));
             if(!event->region().intersects(rect))
             {
                 return;
             }
             QColor color = white ? Qt::white : Qt::black;
-            QColor border = white ? Qt::black : Qt::white;
+            QColor border = white ? Qt::black : Qt::gray;
             p.setPen(border);
             p.setBrush(QColor(color));
             p.drawRect(rect);
@@ -262,7 +269,6 @@ void BoardView::drawMoveIndicator(QPaintEvent* event)
 void BoardView::drawHiliteSquare(QPoint pos, BoardTheme::ColorRole role)
 {
     QPainter p(this);
-    p.translate(m_translate);
 
     QPen pen;
     pen.setColor(m_theme.color(role));
@@ -273,15 +279,6 @@ void BoardView::drawHiliteSquare(QPoint pos, BoardTheme::ColorRole role)
                m_theme.size().width() - 2 - m_showFrame, m_theme.size().height() - 2 - m_showFrame);
 }
 
-QPoint BoardView::posFromSquare(int square) const
-{
-    int coord =  m_coordinates ? CoordinateSize : 0;
-    int x = isFlipped() ? 7 - square % 8 : square % 8;
-    int y = isFlipped() ? square / 8 : 7 - square / 8;
-    QPoint pos(coord + x * m_theme.size().width(), y * m_theme.size().height());
-    return pos;
-}
-
 void BoardView::drawHiliting(QPaintEvent* event)
 {
     for (Square square=a1; square<NumSquares; ++square)
@@ -336,11 +333,35 @@ void BoardView::drawHiliting(QPaintEvent* event)
     }
 }
 
+void BoardView::drawTargets(QPaintEvent* event)
+{
+    for (Square square=a1; square<NumSquares; ++square)
+    {
+        if (m_targets.contains(square))
+        {
+            drawColorRect(event, square, m_theme.color(BoardTheme::Target));
+        }
+    }
+}
+
+void BoardView::drawCheck(QPaintEvent* event)
+{
+    if (isEnabled())
+    {
+        QPainter p(this);
+
+        QRect rect = squareRect(m_alertSquare);
+        if(event->region().intersects(rect))
+        {
+            drawColorRect(event, m_alertSquare, m_theme.color(BoardTheme::Check));
+        }
+    }
+}
+
 void BoardView::drawPieces(QPaintEvent* event)
 {
     QPainter p(this);
-    p.translate(m_translate);
-
+    p.setRenderHint(QPainter::SmoothPixmapTransform);
     for (Square square=a1; square<NumSquares; ++square)
     {
         QRect rect = squareRect(square);
@@ -370,8 +391,8 @@ void BoardView::drawPieces(QPaintEvent* event)
     }
 
     // Always draw outer rectangle
-    QRect rect1 = squareRectNoTranslate(a8);
-    QRect rect2 = squareRectNoTranslate(h1);
+    QRect rect1 = squareRect(a8);
+    QRect rect2 = squareRect(h1);
     p.setPen(m_theme.color(BoardTheme::Frame));
     p.drawRect(!isFlipped() ? QRect(rect1.topLeft(),rect2.bottomRight()) :
                               QRect(rect2.topLeft(),rect1.bottomRight()));
@@ -382,6 +403,8 @@ void BoardView::paintEvent(QPaintEvent* event)
     drawSquares(event);
     drawCoordinates(event);
     drawSquareAnnotations(event);
+    drawTargets(event);
+    drawCheck(event);
     drawPieces(event);
     drawHiliting(event);
     drawMoveIndicator(event);
@@ -404,13 +427,12 @@ void BoardView::resizeBoard(QSize sz)
 
     QSize widgetSize(size * 8 + 1 + coord + square, size * 8 + 1 + coord);
     QPoint widgetCenter(widgetSize.width() / 2, widgetSize.height() / 2);
-    m_translate = geometry().center() - widgetCenter - geometry().topLeft();
-    if (m_vAlignTop) m_translate.setY(0);
 }
 
 void BoardView::resizeEvent(QResizeEvent* e)
 {
     resizeBoard(e->size());
+    QWidget::resizeEvent(e);
 }
 
 QSize BoardView::sizeHint() const
@@ -443,8 +465,8 @@ int BoardView::moveIndicatorWidth(int width, int themeWidth) const
 
 Square BoardView::squareAt(const QPoint& p) const
 {
-    int x = p.x() - m_translate.x();
-    int y = p.y() - m_translate.y();
+    int x = p.x();
+    int y = p.y();
     int width = m_theme.size().width();
     int height = m_theme.size().height();
     x -= m_coordinates ? CoordinateSize : 0;
@@ -467,7 +489,7 @@ bool BoardView::showGuess(Square s)
 {
     // Don't want to constantly recalculate guess, so remember which square
     // the mouse is hovering over, and only show new guess when it changes
-    if(m_guessMove && s != InvalidSquare && s != m_hoverSquare && !(m_flags & SuppressGuessMove))
+    if((m_guessMove || m_showTargets) && s != InvalidSquare && s != m_hoverSquare && !(m_flags & SuppressGuessMove))
     {
         removeGuess();
         m_hoverSquare = s;
@@ -477,10 +499,26 @@ bool BoardView::showGuess(Square s)
             Guess::Result sm = Guess::guessMove(qPrintable(m_board.toFen()), m_board.chess960(), m_board.castlingRooks(), (Guess::squareT)s, m_moveList);
             if(!sm.error)
             {
-                m_hiFrom = Square(sm.from);
-                m_hiTo = Square(sm.to);
-                update(squareRect(m_hiFrom));
-                update(squareRect(m_hiTo));
+                if (m_guessMove)
+                {
+                    m_hiFrom = Square(sm.from);
+                    m_hiTo = Square(sm.to);
+                    update(squareRect(m_hiFrom));
+                    update(squareRect(m_hiTo));
+                }
+
+                if (m_showTargets)
+                {
+                    foreach(Guess::simpleMoveT sm, m_moveList)
+                    {
+                        if (s == Square(sm.from))
+                        {
+                            Square target = Square(sm.visualTo());
+                            m_targets.append(target);
+                            update(squareRect(target));
+                        }
+                    }
+                }
             }
         }
         return true;
@@ -542,6 +580,11 @@ void BoardView::removeGuess()
     m_hoverSquare = InvalidSquare;
     m_moveListCurrent = 0;
     m_moveList.clear();
+    foreach(Square s, m_targets)
+    {
+        update(squareRect(s));
+    }
+    m_targets.clear();
 }
 
 void BoardView::nextGuess(Square s)
@@ -586,6 +629,8 @@ BoardView::BoardViewAction BoardView::moveActionFromModifier(Qt::KeyboardModifie
         return m_atLineEnd ? ActionStandard : ActionInsert;
     case MetaModifier:
         return ActionQuery;
+    case (unsigned int)ControlModifier | (unsigned int)ShiftModifier:
+        return ActionAskEngine;
     default:
         return ActionStandard;
     }
@@ -620,6 +665,10 @@ void BoardView::checkCursor(Qt::KeyboardModifiers modifiers)
         file = ":/images/pen.png";
         text = tr("Draw a square or arrow annotation");
         break;
+    case ActionAskEngine:
+        file = ":/images/engine.png";
+        text = tr("Query the engine while hovering the piece");
+        break;
     }
 
     if(file)
@@ -663,9 +712,11 @@ void BoardView::mouseMoveEvent(QMouseEvent *event)
 {
     setFocus();
 
+    Qt::KeyboardModifiers mdf = event->modifiers();
+
     if(!(event->buttons() & Qt::LeftButton))
     {
-        if(!(event->modifiers() & Qt::ShiftModifier))
+        if(!(mdf & Qt::ShiftModifier))
         {
             showGuess(squareAt(event->pos()));
         }
@@ -682,10 +733,14 @@ void BoardView::mouseMoveEvent(QMouseEvent *event)
         m_dragPoint = event->pos() - m_theme.pieceCenter();
         update(old);
         update(QRect(m_dragPoint, m_theme.size()));
+        if (moveActionFromModifier(mdf) == ActionAskEngine)
+        {
+            emit evalRequest(m_dragStartSquare, squareAt(event->pos()));
+        }
         return;
     }
 
-    if(event->modifiers() & Qt::ShiftModifier)
+    if(mdf & Qt::ShiftModifier && (moveActionFromModifier(mdf) != ActionAskEngine))
     {
         return;
     }
@@ -711,7 +766,7 @@ void BoardView::mouseMoveEvent(QMouseEvent *event)
 void BoardView::startToDrag(QMouseEvent *event, Square s)
 {
     removeGuess();
-    m_dragged = m_board.pieceAt(s);
+    if (!m_brushMode) m_dragged = m_board.pieceAt(s);
     m_dragPoint = event->pos() - m_theme.pieceCenter();
     m_dragStartSquare = s;
     update(squareRect(s));
@@ -719,6 +774,16 @@ void BoardView::startToDrag(QMouseEvent *event, Square s)
     unselectSquare();
 }
 
+bool BoardView::getBrushMode() const
+{
+    return m_brushMode;
+}
+
+void BoardView::setBrushMode(bool brushMode)
+{
+    m_brushMode = brushMode;
+}
+
 void BoardView::mouseReleaseEvent(QMouseEvent* event)
 {
     int button = event->button() + event->modifiers();
@@ -746,6 +811,17 @@ void BoardView::mouseReleaseEvent(QMouseEvent* event)
     }
     else
     {
+        if (moveActionFromModifier(event->modifiers()) == ActionAskEngine)
+        {
+            Square from = squareAt(m_dragStart);
+            m_dragStartSquare = InvalidSquare;
+            QRect oldr = QRect(m_dragPoint, m_theme.size());
+            m_dragged = Empty;
+            update(squareRect(from));
+            update(oldr);
+            emit evalModeDone();
+            return;
+        }
         if(event->modifiers() & Qt::ShiftModifier)
         {
             if(s != InvalidSquare)
@@ -758,7 +834,16 @@ void BoardView::mouseReleaseEvent(QMouseEvent* event)
         }
     }
 
-    if(m_dragged != Empty)
+    if(m_brushMode)
+    {
+        if(s != InvalidSquare)
+        {
+            Square from = squareAt(m_dragStart);
+            emit clicked(s, button, mapToGlobal(event->pos()), from);
+            m_dragged = Empty;
+        }
+    }
+    else if(m_dragged != Empty)
     {
         Square from = squareAt(m_dragStart);
         m_dragStartSquare = InvalidSquare;
@@ -837,14 +922,15 @@ void BoardView::wheelEvent(QWheelEvent* e)
 
 void BoardView::setFlipped(bool flipped)
 {
+    bool wasFlipped = m_flipped;
     m_flipped = flipped;
     update();
+    emit signalFlipped(wasFlipped, m_flipped);
 }
 
 void BoardView::flip()
 {
-    m_flipped = !m_flipped;
-    update();
+    setFlipped(!m_flipped);
 }
 
 bool BoardView::isFlipped() const
@@ -859,6 +945,7 @@ void BoardView::configure()
     m_coordinates = AppSettings->getValue("showCoordinates").toBool();
     m_showCurrentMove = AppSettings->getValue("showCurrentMove").toInt();
     m_guessMove = AppSettings->getValue("guessMove").toBool();
+    m_showTargets = AppSettings->getValue("showTargets").toBool();
     m_showThreat = AppSettings->getValue("showThreat").toBool();
     m_minDeltaWheel = AppSettings->getValue("minWheelCount").toInt();
     m_showMoveIndicatorMode = AppSettings->getValue("showMoveIndicator").toInt();
@@ -900,17 +987,16 @@ void BoardView::unselectSquare()
     }
 }
 
-QRect BoardView::squareRect(Square square) const
+QPoint BoardView::posFromSquare(int square) const
 {
     int coord =  m_coordinates ? CoordinateSize : 0;
     int x = isFlipped() ? 7 - square % 8 : square % 8;
     int y = isFlipped() ? square / 8 : 7 - square / 8;
-    return QRect(QPoint(x * m_theme.size().width() + coord,
-                        y * m_theme.size().height()) + m_translate,
-                 m_theme.size());
+    QPoint pos(coord + x * m_theme.size().width(), y * m_theme.size().height());
+    return pos;
 }
 
-QRect BoardView::squareRectNoTranslate(Square square) const
+QRect BoardView::squareRect(Square square) const
 {
     return QRect(posFromSquare(square), m_theme.size());
 }
@@ -919,7 +1005,7 @@ QRect BoardView::coordinateRectVertical(int n) const
 {
     Q_ASSERT(m_coordinates);
     int x = isFlipped() ? n % 8 : 7 - n % 8;
-    return QRect(QPoint(0, x * m_theme.size().height() + (m_theme.size().height() - CoordinateSize) / 2) + m_translate,
+    return QRect(QPoint(0, x * m_theme.size().height() + (m_theme.size().height() - CoordinateSize) / 2),
                  QSize(CoordinateSize, CoordinateSize));
 }
 
@@ -928,7 +1014,7 @@ QRect BoardView::coordinateRectHorizontal(int n) const
     Q_ASSERT(m_coordinates);
     int y = isFlipped() ? 7 - n % 8 : n % 8;
     return QRect(QPoint(CoordinateSize + (y * (m_theme.size().width())) + (m_theme.size().width() - CoordinateSize) / 2,
-                        8 * m_theme.size().height()) + m_translate,
+                        8 * m_theme.size().height()),
                  QSize(CoordinateSize, CoordinateSize));
 }
 
@@ -1021,10 +1107,41 @@ void BoardView::drawArrowAnnotations(QPaintEvent* event)
     }
 }
 
-void BoardView::drawSquareAnnotation(QPaintEvent* event, QString annotation)
+void BoardView::drawColorRect(QPaintEvent* event, Square square, QColor color)
 {
     QPainter p(this);
-    p.translate(m_translate);
+
+    QRect rect = squareRect(square);
+    if(!event->region().intersects(rect))
+    {
+        return;
+    }
+
+    int x = isFlipped() ? 7 - square % 8 : square % 8;
+    int y = isFlipped() ? square / 8 : 7 - square / 8;
+    QPoint pos(x * m_theme.size().width(), y * m_theme.size().height());
+
+    p.save();
+    QPen pen(color);
+    QBrush brush(color);
+    p.setPen(pen);
+    p.setBrush(brush);
+    p.setOpacity(.35);
+
+    int coord =  m_coordinates ? CoordinateSize : 0;
+
+    QRadialGradient radialGrad(QPointF(coord + pos.x()+m_theme.size().width()/2, pos.y()+m_theme.size().height()/2), m_theme.size().width());
+    radialGrad.setColorAt(0, color.lighter());
+    radialGrad.setColorAt(0.5, color);
+    radialGrad.setColorAt(1, color);
+
+    p.fillRect(coord + pos.x(), pos.y(), m_theme.size().width(), m_theme.size().height(), radialGrad);
+
+    p.restore();
+}
+
+void BoardView::drawSquareAnnotation(QPaintEvent* event, QString annotation)
+{
     QString trimmed = annotation.simplified();
     QChar colorChar = trimmed[0];
     QChar fileChar = trimmed[1];
@@ -1033,17 +1150,6 @@ void BoardView::drawSquareAnnotation(QPaintEvent* event, QString annotation)
     QString ranks = "12345678";
     int file = files.indexOf(fileChar);
     int rank = ranks.indexOf(rankChar);
-    Square square = SquareFromRankAndFile(rank, file);
-
-    QRect rect = squareRect(square);
-    if(!event->region().intersects(rect))
-    {
-        return;
-    }
-    int x = isFlipped() ? 7 - square % 8 : square % 8;
-    int y = isFlipped() ? square / 8 : 7 - square / 8;
-    QPoint pos(x * m_theme.size().width(), y * m_theme.size().height());
-    //p.drawPixmap(pos, m_theme.square((x + y) % 2));
 
     QColor color = Qt::red;
     if(colorChar == 'Y')
@@ -1059,25 +1165,13 @@ void BoardView::drawSquareAnnotation(QPaintEvent* event, QString annotation)
         color = Qt::blue;
     }
 
-    p.save();
-    QPen pen(color);
-    QBrush brush(color);
-    p.setPen(pen);
-    p.setBrush(brush);
-
-    p.setOpacity(.35);
-
-    int coord =  m_coordinates ? CoordinateSize : 0;
-    p.drawRect(coord + pos.x(), pos.y(), m_theme.size().width(), m_theme.size().height());
-
-    p.restore();
-
+    Square square = SquareFromRankAndFile(rank, file);
+    drawColorRect(event, square, color);
 }
 
 void BoardView::drawArrow(int square1, int square2, QColor color)
 {
     QPainter p(this);
-    p.translate(m_translate);
 
     int x1 = isFlipped() ? 7 - square1 % 8 : square1 % 8;
     int y1 = isFlipped() ? square1 / 8 : 7 - square1 / 8;
@@ -1209,18 +1303,6 @@ void BoardView::setDragged(const Piece &dragged)
     m_dragged = dragged;
 }
 
-
-bool BoardView::vAlignTop() const
-{
-    return m_vAlignTop;
-}
-
-void BoardView::setVAlignTop(bool vAlignTop)
-{
-    m_vAlignTop = vAlignTop;
-}
-
-
 void BoardView::setEnabled(bool enabled)
 {
     QWidget::setEnabled(enabled);
