@@ -37,7 +37,7 @@ public:
     enum {WheelUp = Qt::LeftButton, WheelDown = Qt::RightButton};
     enum {Automatic = 0, Always = 1, Never = 2};
     enum {IgnoreSideToMove = 1, SuppressGuessMove = 2, AllowCopyPiece = 4};
-    typedef enum {ActionStandard, ActionQuery, ActionReplace, ActionInsert, ActionAdd, ActionPen } BoardViewAction;
+    typedef enum {ActionStandard, ActionQuery, ActionReplace, ActionInsert, ActionAdd, ActionPen, ActionAskEngine } BoardViewAction;
     /** Create board widget. */
     BoardView(QWidget* parent = 0, int flags = 0);
     /** Destroy widget. */
@@ -88,6 +88,10 @@ public:
     QRect totalRect() const;
 
     BoardViewAction moveActionFromModifier(Qt::KeyboardModifiers modifiers);
+    bool getBrushMode() const;
+    void setBrushMode(bool brushMode);
+
+    QSize themeSize() const;
 public slots:
 
     /** Flips/unflips board. */
@@ -106,6 +110,9 @@ public slots:
 signals:
     /** User clicked source and destination squares */
     void moveMade(Square from, Square to, int button);
+    /** User requests an evaluation from the current position with the piece @p from replaced at @p to */
+    void evalRequest(Square from, Square to);
+    void evalModeDone();
     /** User dragged and dropped a piece holding Control */
     void copyPiece(Square from, Square to);
     /** User dragged and dropped a piece holding Control */
@@ -117,6 +124,8 @@ signals:
     /** Indicate that a piece was dropped to the board */
     void pieceDropped(Square to, Piece p);
     void actionHint(const QString&);
+    void signalFlipped(bool oldState, bool newState);
+
 protected:
     /** Redraws whole board if necessary. */
     virtual void paintEvent(QPaintEvent*);
@@ -149,7 +158,6 @@ private:
     void resizeBoard(QSize size);
     /** Calculate size and position of square */
     QRect squareRect(Square s) const;
-    QRect squareRectNoTranslate(Square s) const;
     /** Calculate size and position of a coordinate indicator in vertical direction */
     QRect coordinateRectVertical(int n) const;
     /** Calculate size and position of a coordinate indicator in horizontal direction */
@@ -180,9 +188,13 @@ private:
 
     QPoint posFromSquare(int square) const;
 
+    void drawColorRect(QPaintEvent* event, Square square, QColor color);
+
     void drawHiliting(QPaintEvent* event);
     void drawSquares(QPaintEvent* event);
+    void drawTargets(QPaintEvent* event);
     void drawPieces(QPaintEvent* event);
+    void drawCheck(QPaintEvent* event);
     void drawMoveIndicator(QPaintEvent* event);
     void drawDraggedPieces(QPaintEvent* event);
     void drawCoordinates(QPaintEvent* event);
@@ -196,12 +208,13 @@ private:
 
     Board m_board;
     BoardTheme m_theme;
-    QPoint m_translate;
     bool m_flipped;
     bool m_showFrame;
     int m_showCurrentMove;
     bool m_guessMove;
     bool m_showThreat;
+    bool m_showTargets;
+    bool m_brushMode;
     Square m_selectedSquare;
     Square m_hoverSquare;
     Square m_hiFrom;
@@ -213,6 +226,8 @@ private:
     Square m_storedFrom;
     Square m_storedTo;
     Square m_dragStartSquare;
+    Square m_alertSquare;
+    QList<Square> m_targets;
     bool m_atLineEnd;
     int m_flags;
     bool m_coordinates;
@@ -225,7 +240,6 @@ private:
     Guess::MoveList m_moveList;
     unsigned int m_moveListCurrent;
     bool m_showMoveIndicator;
-    bool m_vAlignTop;
     int m_showMoveIndicatorMode;
     QPointer<QObject> m_DbIndex;
     ThreadedGuess m_threatGuess;
