@@ -32,6 +32,7 @@
 #include "gamewindow.h"
 #include "helpbrowser.h"
 #include "helpbrowsershell.h"
+#include "historylabel.h"
 #include "kbaction.h"
 #include "loadquery.h"
 #include "mainwindow.h"
@@ -92,7 +93,8 @@ MainWindow::MainWindow() : QMainWindow(),
     m_lastColor('G'),
     m_machineHasToMove(false),
     m_gameMode(false),
-    m_scratchPad(0)
+    m_scratchPad(0),
+    m_bEvalRequested(false)
 {
     setObjectName("MainWindow");
 
@@ -164,12 +166,13 @@ MainWindow::MainWindow() : QMainWindow(),
     connect(m_ficsConsole, SIGNAL(ReceivedBoard(int,QString)), this, SLOT(HandleFicsBoardRequest(int,QString)));
     connect(m_ficsConsole, SIGNAL(SignalGameResult(QString)), this, SLOT(HandleFicsResultRequest(QString)));
     connect(m_ficsConsole, SIGNAL(RequestNewGame()), this, SLOT(HandleFicsNewGameRequest()));
+    connect(m_ficsConsole, SIGNAL(FicsShowTimer(bool)), this, SLOT(SlotShowTimer(bool)));
+    connect(m_ficsConsole, SIGNAL(FicsShowTime(int, QString)), this, SLOT(SlotDisplayTime(int,QString)));
     connect(m_ficsConsole, SIGNAL(RequestSaveGame()), this, SLOT(HandleFicsSaveGameRequest()));
     connect(m_ficsConsole, SIGNAL(RequestCloseFICS()), this, SLOT(HandleFicsCloseRequest()));
     connect(m_ficsConsole, SIGNAL(RequestAddTag(QString,QString)), this, SLOT(HandleFicsAddTagRequest(QString,QString)));
     connect(m_ficsConsole, SIGNAL(RequestGameMode(bool)), this, SLOT(enterGameMode(bool)));
     connect(m_ficsConsole, SIGNAL(RequestRemoveLastMove()), this, SLOT(HandleFicsRequestRemoveMove()));
-    connect(m_ficsConsole, SIGNAL(SignalPlayerIsBlack(bool)), this, SLOT(slotFlipView(bool)));
     connect(m_ficsClient, SIGNAL(disconnected()), SLOT(FicsDisconnected()), Qt::QueuedConnection);
     connect(m_ficsClient, SIGNAL(connected()), SLOT(FicsConnected()), Qt::QueuedConnection);
     connect(m_ficsConsole, SIGNAL(RequestStoredMove()), SLOT(slotBoardStoredMove()));
@@ -378,16 +381,19 @@ MainWindow::MainWindow() : QMainWindow(),
     /* Restoring layouts */
     if(!AppSettings->layout(this))
     {
-        resize(800, 600);
+        resize(1400, 600);
+    }
+    else
+    {
+        m_boardSplitter->restoreState(AppSettings->value("/MainWindow/BoardSplit").toByteArray());
     }
-    m_boardSplitter->restoreState(AppSettings->value("/MainWindow/BoardSplit").toByteArray());
 
     /* Status */
     m_statusFilter = new QLabel();
     statusBar()->addPermanentWidget(m_statusFilter,1);
     m_statusFilter->setFrameStyle(QFrame::Panel | QFrame::Sunken);
 
-    m_statusApp = new QLabel();
+    m_statusApp = new HistoryLabel();
     statusBar()->addPermanentWidget(m_statusApp,1);
     m_statusApp->setFrameStyle(QFrame::Panel | QFrame::Sunken);
 
@@ -398,15 +404,20 @@ MainWindow::MainWindow() : QMainWindow(),
 
     m_sliderSpeed = new TranslatingSlider(this);
     m_sliderSpeed->setMultiplier(1000);
+    m_sliderSpeed->setMultiplier2(10000);
     m_sliderSpeed->setOrientation(Qt::Horizontal);
-    m_sliderSpeed->setMinimum(0);
-    m_sliderSpeed->setMaximum(60);
+    m_sliderSpeed->setMinimum(0);  // O = Infinite
+    m_sliderSpeed->setStart2(30);  // Step 10s after 30s
+    m_sliderSpeed->setMaximum(87); // 10 Minutes
     m_sliderSpeed->setTranslatedValue(AppSettings->getValue("/Board/AutoPlayerInterval").toInt());
     m_sliderSpeed->setTickInterval(1);
     m_sliderSpeed->setTickPosition(QSlider::NoTicks);
-    m_sliderSpeed->setMaximumWidth(400);
-    connect(m_sliderSpeed, SIGNAL(translatedValueChanged(int)), SLOT(slotMoveIntervalChanged(int)));
+    m_sliderSpeed->setSingleStep(1);
+    m_sliderSpeed->setPageStep(1);
+    m_sliderSpeed->setMinimumWidth(100); // 87 + some pixel for overlapping slider
+    m_sliderSpeed->setMaximumWidth(400); // Arbitrary limit - not really needed
 
+    connect(m_sliderSpeed, SIGNAL(translatedValueChanged(int)), SLOT(slotMoveIntervalChanged(int)));
     connect(m_mainAnalysis, SIGNAL(receivedBestMove(const Analysis&)), this, SLOT(slotEngineTimeout(const Analysis&)));
     connect(m_secondaryAnalysis, SIGNAL(receivedBestMove(const Analysis&)), this, SLOT(slotEngineTimeout(const Analysis&)));
 
@@ -431,8 +442,8 @@ MainWindow::MainWindow() : QMainWindow(),
     m_sliderText->setFixedWidth(m_sliderText->sizeHint().width());
 
     statusBar()->addPermanentWidget(m_sliderText);
-    connect(m_sliderSpeed, SIGNAL(valueChanged(int)), this, SLOT(slotSetSliderText(int)));
-    slotSetSliderText(m_sliderSpeed->value());
+    connect(m_sliderSpeed, SIGNAL(translatedValueChanged(int)), this, SLOT(slotSetSliderText(int)));
+    slotSetSliderText(m_sliderSpeed->translatedValue());
 
     statusBar()->setFixedHeight(statusBar()->height());
     statusBar()->setSizeGripEnabled(true);
@@ -745,7 +756,7 @@ DatabaseInfo* MainWindow::getDatabaseInfoByPath(QString path)
 {
     for(int i = 0; i < m_databases.count(); ++i)
     {
-        if(m_databases[i]->filePath() == path)
+        if(m_databases[i]->displayName() == path)
         {
             return m_databases[i];
         }
@@ -896,6 +907,7 @@ void MainWindow::openDatabaseUrl(QString fname, bool utf8)
         if (url.isValid() && !url.isRelative() &&
             ((url.scheme() == "http") || (url.scheme() == "https") || (url.scheme() == "ftp")))
         {
+            slotStatusMessage(tr("Start loading database..."));
             connect(downloadManager, SIGNAL(downloadError(QUrl)), this, SLOT(loadError(QUrl)), static_cast<Qt::ConnectionType>(Qt::QueuedConnection | Qt::UniqueConnection));
             connect(downloadManager, SIGNAL(onDownloadFinished(QUrl, QString)), this, SLOT(loadReady(QUrl, QString)), static_cast<Qt::ConnectionType>(Qt::QueuedConnection | Qt::UniqueConnection));
             downloadManager->doDownload(url);
@@ -1086,7 +1098,8 @@ void MainWindow::loadError(QUrl url)
 void MainWindow::loadReady(QUrl url, QString fileName)
 {
     QString fav = favoriteUrl();
-    if ((url == fav) && fav.contains("$1"))
+    QString favurl = AppSettings->getValue("Web/Favorite1").toString();
+    if ((url == fav) && favurl.contains("$1"))
     {
         int n = AppSettings->getValue("Web/AutoNumber1").toInt();
         ++n;
@@ -1104,7 +1117,7 @@ void MainWindow::slotDataBaseLoaded(DatabaseInfo* db)
         delete db;
         return;
     }
-    QString fname = db->filePath();
+    QString fname = db->displayName();
     QFileInfo fi = QFileInfo(fname);
     QString basefile = fi.completeBaseName();
 
@@ -1408,6 +1421,30 @@ void MainWindow::setupActions()
     edit->addAction(createAction(tr("Paste"), SLOT(slotEditMergePGN()),
                                  Qt::CTRL + Qt::Key_V, editToolBar, ":/images/edit_paste.png"));
     edit->addSeparator();
+    brushGroup = new ExclusiveActionGroup(this);
+    QMenu* brush = edit->addMenu(tr("Brush"));
+    QAction* brushAction = createAction(tr("Green"), SLOT(slotToggleBrush()), Qt::META + Qt::Key_1, editToolBar, ":/images/brush_green.png");
+    brushAction->setCheckable(true);
+    brushAction->setData(QChar('G'));
+    brush->addAction(brushAction);
+    brushGroup->addAction(brushAction);
+    brushAction = createAction(tr("Yellow"), SLOT(slotToggleBrush()), Qt::META + Qt::Key_2, editToolBar, ":/images/brush_yellow.png");
+    brushAction->setCheckable(true);
+    brushAction->setData(QChar('Y'));
+    brush->addAction(brushAction);
+    brushGroup->addAction(brushAction);
+    brushAction = createAction(tr("Red"), SLOT(slotToggleBrush()), Qt::META + Qt::Key_3, editToolBar, ":/images/brush_red.png");
+    brushAction->setCheckable(true);
+    brushAction->setData(QChar('R'));
+    brush->addAction(brushAction);
+    brushGroup->addAction(brushAction);
+    brushAction = createAction(tr("Erase"), SLOT(slotToggleBrush()), Qt::META + Qt::Key_4, editToolBar, ":/images/brush_erase.png");
+    brushAction->setCheckable(true);
+    brushAction->setData(QChar(0));
+    brush->addAction(brushAction);
+    brushGroup->addAction(brushAction);
+    edit->addSeparator();
+
     edit->addAction(createAction(tr("&Preferences..."), SLOT(slotConfigure()), QKeySequence(), 0,
                                  QString(), QString(), QAction::PreferencesRole));
 
@@ -1431,6 +1468,22 @@ void MainWindow::setupActions()
                                        0, style()->standardIcon(QStyle::SP_TitleBarCloseButton)));
     m_menuView->addSeparator();
 
+    QToolBar* viewToolBar = addToolBar(tr("View"));
+    viewToolBar->setObjectName("ViewToolBarMain");
+    QAction* showTargets = createAction(tr("Show target fields"), SLOT(slotShowTargetFields()), 0,
+                                       viewToolBar, QIcon(":/images/show_targets.png"));
+    m_menuView->addAction(showTargets);
+    showTargets->setCheckable(true);
+    showTargets->setChecked(AppSettings->getValue("/Board/showTargets").toBool());
+
+    QAction* showThreat = createAction(tr("Show threat"), SLOT(slotShowThreat()), 0,
+                                       viewToolBar, QIcon(":/images/show_threat.png"));
+    m_menuView->addAction(showThreat);
+    showThreat->setCheckable(true);
+    showThreat->setChecked(AppSettings->getValue("/Board/showThreat").toBool());
+
+    m_menuView->addSeparator();
+
     /* Game menu */
     QMenu *gameMenu = menuBar()->addMenu(tr("&Game"));
     QToolBar* gameToolBar = addToolBar(tr("Game"));
@@ -1453,6 +1506,7 @@ void MainWindow::setupActions()
                                         dbToolBar, ":/images/game_down.png");
     connect(m_gameList, SIGNAL(signalLastGameLoaded(bool)), nextAction, SLOT(setDisabled(bool)));
     connect(m_gameList, SIGNAL(signalFirstGameLoaded(bool)), prevAction, SLOT(setDisabled(bool)));
+    connect(m_gameList, SIGNAL(signalDropEvent(QDropEvent*)), this, SLOT(slotDatabaseDropped(QDropEvent*)));
     loadMenu->addAction(nextAction);
     loadMenu->addAction(prevAction);
     loadMenu->addAction(createAction(tr("&Go to game..."), SLOT(slotGameLoadChosen()), Qt::CTRL + Qt::Key_G));
@@ -1469,20 +1523,20 @@ void MainWindow::setupActions()
 
     QAction* flip = createAction(tr("&Flip board"), SLOT(slotConfigureFlip()), Qt::CTRL + Qt::Key_B, gameToolBar, ":/images/flip_board.png");
     flip->setCheckable(true);
-    connect(m_ficsConsole, SIGNAL(SignalPlayerIsBlack(bool)), flip, SLOT(setChecked(bool)));
+    connect(m_ficsConsole, SIGNAL(SignalPlayerIsBlack(bool,bool)), flip, SLOT(setChecked(bool)));
     gameToolBar->addSeparator();
 
     autoGroup = new ExclusiveActionGroup(this);
     autoGroupTraining = new ExclusiveActionGroup(this);
 
-    QAction* match = createAction(tr("Match"), SLOT(slotToggleGameMode()), Qt::CTRL + Qt::Key_M, gameToolBar, ":/images/black_chess.png");
-    match->setCheckable(true);
+    m_match = createAction(tr("Match"), SLOT(slotToggleGameMode()), Qt::CTRL + Qt::Key_M, gameToolBar, ":/images/black_chess.png");
+    m_match->setCheckable(true);
 
-    autoGroupTraining->addAction(match);
-    autoGroup->addAction(match);
-    gameMenu->addAction(match);
+    autoGroupTraining->addAction(m_match);
+    autoGroup->addAction(m_match);
+    gameMenu->addAction(m_match);
     gameMenu->addSeparator();
-    connect(match, SIGNAL(changed()), SLOT(slotToggleGameMode()));
+    connect(m_match, SIGNAL(changed()), SLOT(slotToggleGameMode()));
 
     m_training = createAction(tr("Training"), SLOT(slotToggleTraining()), Qt::CTRL + Qt::Key_R, gameToolBar, ":/images/training.png");
     m_training->setCheckable(true);
@@ -1663,6 +1717,7 @@ void MainWindow::setupActions()
     toolbars->addAction(dbToolBar->toggleViewAction());
     toolbars->addAction(gameToolBar->toggleViewAction());
     toolbars->addAction(searchToolBar->toggleViewAction());
+    toolbars->addAction(viewToolBar->toggleViewAction());
 
     KbAction::restoreKeyboardLayoutForObject(this);
 }
@@ -1878,7 +1933,6 @@ void MainWindow::QueryLoadDatabase()
     LoadQuery dlg(this);
     if(dlg.exec() == QDialog::Accepted)
     {
-        slotStatusMessage(tr("Start loading database..."));
         if(dlg.largeDB())
         {
             openDatabaseUrl("http://chessx.sourceforge.net/db/bundesliga2000.pgn.zip", false);
@@ -1911,10 +1965,12 @@ void MainWindow::slotVersionFound(int major, int minor, int build)
 
 void MainWindow::dragEnterEvent(QDragEnterEvent *event)
 {
+    m_pDragTabBar = 0;
+    m_tabDragIndex = -1;
     const QMimeData *mimeData = event->mimeData();
     const GameMimeData* gameMimeData = qobject_cast<const GameMimeData*>(mimeData);
     const DbMimeData* dbMimeData = qobject_cast<const DbMimeData*>(mimeData);
-    if(mimeData->hasUrls() || gameMimeData || dbMimeData)
+    if((mimeData && mimeData->hasUrls()) || gameMimeData || dbMimeData)
     {
         event->acceptProposedAction();
     }
@@ -1938,7 +1994,7 @@ void MainWindow::dragMoveEvent(QDragMoveEvent *event)
         m_dragTimer->stop();
         m_tabDragIndex = tabIndex;
         m_pDragTabBar = pTabBar;
-        m_dragTimer->start(500);
+        m_dragTimer->start(200);
     }
     if (tabIndex == -1)
     {
@@ -1952,12 +2008,16 @@ void MainWindow::dragMoveEvent(QDragMoveEvent *event)
 
 void MainWindow::dragLeaveEvent(QDragLeaveEvent *event)
 {
+    m_pDragTabBar = 0;
+    m_tabDragIndex = -1;
     m_dragTimer->stop();
     event->accept();
 }
 
 void MainWindow::dropEvent(QDropEvent *event)
 {
+    m_pDragTabBar = 0;
+    m_tabDragIndex = -1;
     m_dragTimer->stop();
     event->ignore();
 }
@@ -1981,6 +2041,9 @@ bool MainWindow::gameMode() const
 void MainWindow::setGameMode(bool gameMode)
 {
     m_gameMode = gameMode;
+
+    if (gameMode) brushGroup->untrigger();
+
     emit signalGameModeChanged(m_gameMode);
 
     emit signalMoveHasNextMove(!gameMode && !game().atLineEnd());
