@@ -23,6 +23,7 @@ class Analysis;
 class AnalysisWidget;
 class Board;
 class BoardView;
+class BoardViewEx;
 class ChessBrowser;
 class Database;
 class DatabaseInfo;
@@ -39,6 +40,7 @@ class FicsConsole;
 class Game;
 class GameList;
 class GameWindow;
+class HistoryLabel;
 class OpeningTreeWidget;
 class PlayerListWidget;
 class QLabel;
@@ -311,6 +313,9 @@ public slots:
     void slotSearchTreeMove(const QModelIndex& index);
     /** Made given move on the board */
     void slotBoardMove(Square from, Square to, int button=0);
+    /** Make a evalution with the current board and piece @p from placed at @p to */
+    void slotEvalRequest(Square from, Square to);
+    void slotResumeBoard();
     /** Board square was clicked */
     void slotBoardClick(Square square, int button, QPoint pos, Square from);
     /** Browse current game by mouse wheel */
@@ -384,6 +389,12 @@ public slots:
     void HandleFicsAddTagRequest(QString tag,QString value);
     /** Close FICS upon request from the console */
     void HandleFicsCloseRequest();
+    /** Show a game timer and reset the time to be displayed */
+    void SlotShowTimer(bool show);
+    /** Set the time to be displayed */
+    void SlotDisplayTime(int color, QString t);
+    /** Accept Drop of database to a subordinate target */
+    void slotDatabaseDropped(QDropEvent*);
 protected slots:
     /** Update recent files menu */
     void updateMenuDatabases();
@@ -440,6 +451,9 @@ protected slots:
     void slotRestartAnalysis();
     void slotBoardStoredMove();   
     void slotLoadRecentGame();
+    void slotToggleBrush();
+    void slotShowTargetFields();
+    void slotShowThreat();
 protected:
     void moveChanged();
     bool pasteFen(QString& errorText, QString fen, bool newGame=false);
@@ -473,6 +487,9 @@ protected:
     void playSound(QString s);
     void doBoardMove(Move m, unsigned int button, Square from, Square to);
     QString favoriteUrl() const;
+    QString drawAnnotation() const;
+    BoardViewEx *BoardViewFrame();
+    void SQAction(QChar c, QAction *action);
 signals:
     /** Re-read configuration. */
     void reconfigure();
@@ -579,6 +596,11 @@ private:
     void triggerBoardMove();
     /** Filter Duplicates in the current database */
     void filterDuplicates(int mode);
+    /** Return true, if a game is drawn by rule */
+    bool gameIsDraw() const;
+
+    /** Determine Color the user is using depending upon different match scenarios */
+    Color UserColor();
 
     /* Dialogs  */
     GameList* m_gameList;
@@ -599,7 +621,7 @@ private:
     QTabWidget* m_tabWidget;
     /* Status */
     QLabel* m_statusFilter;
-    QLabel* m_statusApp;
+    HistoryLabel* m_statusApp;
     QLabel* m_gameTitle;
     /* Menus */
     QMenu* m_menuDatabases;
@@ -634,11 +656,13 @@ private:
     QChar m_lastColor;
     ExclusiveActionGroup* autoGroup;
     ExclusiveActionGroup* autoGroupTraining;
+    ExclusiveActionGroup* brushGroup;
     QAction* m_training;
     QAction* m_autoRespond;
     QAction* m_autoPlay;
     QAction* m_autoAnalysis;
     QAction* m_engineMatch;
+    QAction* m_match;
     QUndoGroup m_undoGroup;
     QNetworkAccessManager* m_manager;
     DownloadManager* downloadManager;
@@ -653,6 +677,7 @@ private:
     bool m_elapsedUserTimeValid;
     EngineParameter m_matchParameter;
     bool m_bTrainigAutoRespond;
+    bool m_bEvalRequested;
 };
 
 #endif
